package
{	
	/*
	 * Timequake Prototype Source Code. Released by @realnoyb on December 2012.
	 * This work is licensed under the Creative Commons Attribution-NonCommercial 3.0 Unported License. 
	 * To view a copy of this license, visit http://creativecommons.org/licenses/by-nc/3.0/.
	 */
	
	import org.flixel.FlxG;
	import org.flixel.FlxGroup;
	import org.flixel.FlxObject;
	import org.flixel.FlxSprite;

	public class DumbEnemy extends Gunman
	{
		[Embed(source="../data/player.png")] protected var ImgEnemy:Class;
		
		public static const ENEMY_WALK_SPEED:Number = 50;
		public var dying:Boolean;
		private var text:Description;
		private var canSeePlayer:Boolean;
		private var canSeePlayerTop:Boolean;
		private var canSeePlayerBottom:Boolean;
		private var state:uint;
		private var delayCounter:Number;
		private var aiType:uint;
		
		public var aboutToAlert:Boolean;
		
		public static const STATE_PATROL:uint = 0;
		public static const STATE_ALERT:uint = 1;		
		public static const STATE_SHOOT:uint = 2;
		public static const STATE_DIE:uint = 3;
		public static const STATE_CAUTIOUS:uint = 4;
		
		public static const DELAY_GUN_RAISE:Number = 0;
		public static const DELAY_GUN_SHOOT:Number = 60;
		public static const DELAY_GUN_DROP:Number = 240;
		
		public static const TYPE_MOVING:uint = 0;
		public static const TYPE_STATIONARY:uint = 1;
		
		public function DumbEnemy(X:Number, Y:Number, bullets:FlxGroup, text:Description=null, type:uint=0, startLeft:Boolean=true)
		{
			super(X, Y, bullets, ImgEnemy, null);
			
			this.text = text;
			
			aiType = type;
			
			if(startLeft)
			{
				facing = FlxSprite.LEFT;
			}
			else
			{
				facing = FlxSprite.RIGHT;
			}
				
			acceleration.y = 50;
			
			canSeePlayer = false;
			canSeePlayerTop = false;
			canSeePlayerBottom = false;
			state = STATE_PATROL;
			delayCounter = 0;
			
			aboutToAlert = false;
		}
		
		override public function update():void
		{
			if(state == STATE_PATROL && canSeePlayer)
			{
				//Go into state of alert when see player
				state = STATE_ALERT;
				play("Idle");
				velocity.x = 0;
				delayCounter = 0;
			}
			else if(state == STATE_PATROL && facing == FlxSprite.LEFT && aiType != TYPE_STATIONARY)
			{
				moveLeft();
			}
			else if(state == STATE_PATROL && facing == FlxSprite.RIGHT && aiType != TYPE_STATIONARY)
			{
				moveRight();
			}
			else if(state == STATE_ALERT && !canSeePlayer)
			{
				//Lower gun if player not seen
				delayCounter++;
				
				if(delayCounter >= DELAY_GUN_DROP)
				{
					if(_curAnim.name == "Idle")
						state = STATE_PATROL;
					else if(isCrouching)
						moveUp();
					else
						perform_action2();
				}
			}
			else if(state == STATE_ALERT && canShoot && (!canSeePlayerTop && canSeePlayerBottom && !isCrouching))
			{
				//Crouch to hit player
				moveDown();
			} 
			else if(state == STATE_ALERT && canShoot && (!canSeePlayerBottom && isCrouching))
			{
				//Stand up to hit player
				moveUp();
			} 
			else if(state == STATE_ALERT && canShoot && (canSeePlayerTop && !isCrouching || canSeePlayerBottom && isCrouching))
			{
				//Shoot at player
				delayCounter++;
				if(delayCounter >= DELAY_GUN_SHOOT)
				{
					perform_action();
					delayCounter = 0;
				}
			}
			else if(state == STATE_ALERT && this._curAnim.name == "Idle")
			{
				// Raise gun after short delay
				delayCounter++;
				
				if(delayCounter >= DumbEnemy.DELAY_GUN_RAISE)
				{
					delayCounter = 0;
					perform_action2();
				}
			}
			
			super.update();	
		}
		
				
		public function activateText():void
		{
			if(text == null)
				return;
				
			FlxG.log("Text:" + text.text);
			
			text.activate(this.x + this.width / 2, this.y);
		}
		
		override public function hitLeft(Contact:FlxObject, Velocity:Number):void
		{
			super.hitLeft(Contact, Velocity);
			
			if(!(Contact is Player))		
				facing = FlxSprite.RIGHT;
		}
		
		override public function hitRight(Contact:FlxObject, Velocity:Number):void
		{
			super.hitLeft(Contact, Velocity);
			if(!(Contact is Player))	
				facing = FlxSprite.LEFT;
		}
		
		override public function die(replay:Boolean=false):void
		{
			FlxG.log("Die!");
			play("Die");
			dying = true;
			state = STATE_DIE;
			
			velocity.y = 0;
			velocity.x = 0;
			
			if(replay)
				activateText();				
		}
		
		public function updateSightLine(top:Boolean, bottom:Boolean):void
		{
			canSeePlayer = top || bottom;
			canSeePlayerTop = top;
			canSeePlayerBottom = bottom;
		}
		
		override public function callback(name:String, frameNumber:uint, frameIndex:uint):void
		{
			if(name == "Die" && frameNumber >= 2)
			{
				this.kill();
				return;
			}
			
			if(name == "LowerGun" && frameNumber >= 3)
			{
				state = STATE_PATROL;
			}
			
			if(name == "LowerGunCrouch" && frameNumber >= 3)
			{
				moveUp();
				return;
			}
			
			super.callback(name, frameNumber, frameIndex);
		}
		
		override public function walkSpeed():Number { return ENEMY_WALK_SPEED; }
		
		public function topY():Number
		{
			if(!isCrouching)
				return y;
			else
				return y - 12;
		}
		
		public function getState():uint { return state; }
		
	}
}