package
{
	import org.flixel.FlxGroup;
	import org.flixel.FlxObject;
	import org.flixel.FlxSprite;

	public class Gunman extends FlxSprite
	{
		
		public const WALK_SPEED:Number = 100.0;
		
		public const BULLET_SPEED:Number = 200.0;
		
		public const JUMP_STRENGTH:Number = 120;
		
		public const GRAVITY:Number = 425;
		
		protected var isWalking:Boolean;
		
		protected var isShooting:Boolean;
		
		protected var isCrouching:Boolean;
		
		protected var canJump:Boolean;
		
		protected var canMove:Boolean;
		
		protected var canShoot:Boolean;
		
		protected var icon:ReplayIcon;
		
		protected var bullets:FlxGroup;
		
		
		public function Gunman(X:Number, Y:Number, bullets:FlxGroup, image:Class, icon:ReplayIcon=null)
		{
			super(X, Y, null);
			loadGraphic(image,true,true,16,32);
			
			//Bounding box
			width = 4;
			offset.x = 5;
			height = 22;
			offset.y = 10;
			
			//Physics
			acceleration.y = GRAVITY;
			//this.maxVelocity.y = 200;
			canJump = false;
			
			//icon
			this.icon = icon;
			
			//bullets
			this.bullets = bullets;
			
			
			//Animations
			addAnimation("Idle", [8,8,8,8,9],5);
			addAnimation("Walk", [0,1,2,3,4,5,6,7], 10);
			addAnimation("Jump", [2]);
			addAnimation("Dead", [3]);
			addAnimation("RaiseGun", [10, 11, 12, 13], 20,false);
			addAnimation("GunRaised", [13]);
			addAnimation("RaiseGunCrouch", [18,23,24,20], 15,false);
			addAnimation("LowerGun", [13, 12, 11, 10], 20,false);
			addAnimation("LowerGunCrouch", [20,24,23,18], 15,false);
			addAnimation("Fire", [13, 14, 15,14,13], 10,false);
			addAnimation("FireCrouched", [20,21,22,21,20], 10,false);
			addAnimation("CrouchNormal", [16,17,18], 20, false);
			addAnimation("CrouchedNormal", [18]);
			addAnimation("GetUpNormal", [18,17,16], 20, false);
			addAnimation("CrouchGun", [14, 19, 20], 20, false);
			addAnimation("CrouchedGun", [20]);
			addAnimation("GetUpGun", [20, 19, 14], 20, false);
			addAnimation("Die", [16,17,18,20], 20,false);
			
			addAnimationCallback(callback);
			
			isWalking = false;
			isShooting = false;
			isCrouching = false;
			canShoot = false;
			canMove = true;
			
			play("Idle");
		}
		
		override public function update():void
		{
			super.update();
			
			if (velocity.y > 5)
				canJump = false;
		}
		
		public function moveLeft():Boolean
		{
			// Can switch directions when not able to move
			if (!canMove && !(_curAnim.name == "Fire" || _curAnim.name == "FireCrouched") && facing == FlxSprite.RIGHT)
			{
				facing = FlxSprite.LEFT;
				return true;
			}
			else if(!canMove)
			{
				return false;
			}

			velocity.x = -1 * walkSpeed();
			isWalking = true;
			facing = FlxSprite.LEFT;
			
			if(canJump)
				play("Walk");
				
			return true;
		}
		
		public function moveRight():Boolean
		{
			// Can switch directions when not able to move
			if (!canMove && !(_curAnim.name == "Fire" || _curAnim.name == "FireCrouched") && facing == FlxSprite.LEFT)
			{
				facing = FlxSprite.RIGHT;
				return true;
			}
			else if(!canMove)
			{
				return false;
			}
			
			velocity.x = walkSpeed();
			isWalking = true;
			facing = FlxSprite.RIGHT;
			
			if(canJump)
				play("Walk");
				
			return true;	
		}
		
		public function moveDown():Boolean
		{
			if (canMove && canJump)
			{
				canMove = false;
				canJump = false;
				play("CrouchNormal");
				return true;			
			}
			else if (!canMove && !isShooting && this._curAnim.name == "GunRaised")
			{
				isShooting = false;
				canShoot = false;
				play("CrouchGun");
				return true;	
			}
			else
			{
				return false;
			}
		}
		
		public function moveUp():Boolean
		{
			if(canJump)
			{
				canJump = false;
				y -= 1;
				velocity.y = -1 * JUMP_STRENGTH;
				play("Jump");
				return true;
			}
			else if (!canMove && !isShooting && this._curAnim.name == "CrouchedNormal")
			{
				canMove = false;
				canJump = false;
				play("GetUpNormal");
				return true;			
			}
			else if (!canMove && !isShooting && this._curAnim.name == "CrouchedGun")
			{
				isShooting = false;
				canShoot = false;
				play("GetUpGun");
				return true;	
			}
			else
			{
				return false;
			}
		}
		
		public function wait():void
		{
			velocity.x = 0;
			isWalking = false;
			
			if(this._curAnim.name == "Walk")
				play("Idle");
		}
		
		public function perform_action():Boolean
		{
			/*if(canJump)
			{
				canJump = false;
				y -= 1;
				velocity.y = -1 * JUMP_STRENGTH;
				play("Jump");
				return true;
			}
			
			else if (!canMove && !isShooting && this._curAnim.name == "CrouchedNormal")
			{
				canMove = false;
				canJump = false;
				play("GetUpNormal");
				return true;			
			}*/
			
			if(canShoot)
			{
				canShoot = false;

				var bullet:FlxSprite = bullets.getFirstDead() as FlxSprite;
				
				bullet.exists = true;
				bullet.dead = false;
				
				if(isCrouching)
				{
					play("FireCrouched");
					bullet.y = this.y + 6;
				}
				else
				{
					play("Fire");
					bullet.y = this.y + 5;
				}
				
				
				
				if(this.facing == FlxSprite.LEFT)
				{
					bullet.x = this.x - 4 + bullet.width;
					bullet.velocity.x = -1 * BULLET_SPEED;
				}
				else
				{
					bullet.x = this.x +7 - bullet.width;
					bullet.velocity.x = BULLET_SPEED;
				}
				
				return true;
			}
			
			return false;
		}
		
		public function perform_action2():Boolean
		{
			if (canMove && canJump)
			{
				canMove = false;
				canJump = false;
				play("RaiseGun");
				return true;			
			}
			else if (!canMove && !isShooting && this._curAnim.name == "CrouchedNormal")
			{
				isShooting = false;
				canShoot = false;
				play("RaiseGunCrouch");
				return true;	
			}
			else if (!canMove && !isShooting && this._curAnim.name == "GunRaised")
			{
				isShooting = false;
				canShoot = false;
				play("LowerGun");
				return true;	
			}
			else if (!canMove && !isShooting && this._curAnim.name == "CrouchedGun")
			{
				isShooting = false;
				canShoot = false;
				play("LowerGunCrouch");
				return true;	
			}
			else
			{
				return false;
			}
			
		}
		
		public function bounce():void
		{
			canJump = false;
			velocity.y = -100;
			play ("Jump");
		}
		
		override public function hitBottom(Contact:FlxObject, Velocity:Number):void
		{
			if(canMove)
				canJump = true;
				
			super.hitBottom(Contact, Velocity);
			
			if(this._curAnim.name == "Jump")
				play("Idle");
		}
		
		public function updateIcon(nextAction:uint):void
		{
			if(icon == null)
				return;
				
			if (nextAction == Replay.ACTION)
				icon.action();
			else if (nextAction == Replay.ACTION2)
				icon.action2();
			else if (!Replay.REPLAY_MOVEMENT)
				icon.wait();
			else if (nextAction == Replay.LEFT)
				icon.moveLeft();
			else if (nextAction == Replay.RIGHT)
				icon.moveRight();
			else if (nextAction == Replay.UP)
				icon.moveUp();
			else if (nextAction == Replay.DOWN)
				icon.moveDown();
			else
				icon.wait();		
				
							
			icon.x = x - offset.x;
			icon.y = y - 16 - offset.y;
			icon.update();
		}
		
		public function callback(name:String, frameNumber:uint, frameIndex:uint):void
		{
			if(name == "RaiseGun" && frameNumber >= 3)
			{
				play("GunRaised");
				canShoot = true;
			}
			else if(name == "RaiseGunCrouch" && frameNumber >= 3)
			{
				play("CrouchedGun");
				canShoot = true;
			}
			else if(name == "LowerGun" && frameNumber >= 3)
			{
				play("Idle");
				canMove = true;
				canJump = true;
			}
			else if(name == "LowerGunCrouch" && frameNumber >= 3)
			{
				play("CrouchedNormal");
				canShoot = false;
			}
			else if(name == "Fire" && frameNumber >= 4)
			{
				play("GunRaised");
				canShoot = true;
			}
			else if(name == "FireCrouched" && frameNumber >= 4)
			{
				play("CrouchedGun");
				canShoot = true;
			}
			else if(name == "CrouchNormal" && frameNumber >= 2)
			{
				play("CrouchedNormal");
				height = 16;
				offset.y = 16;
				y+=6;
				isCrouching = true;
			}
			else if(name == "CrouchGun" && frameNumber >= 2)
			{
				play("CrouchedGun");
				canShoot = true;
				height = 16;
				offset.y = 16;
				y+=6;
				isCrouching = true;
			}
			else if(name == "GetUpNormal" && frameNumber >= 2)
			{
				play("Idle");
				canMove = true;
				canJump = true;
				height = 22;
				offset.y = 10;
				y-=6;
				isCrouching = false;
			}
			else if(name == "GetUpGun" && frameNumber >= 2)
			{
				play("GunRaised");
				canShoot = true;
				height = 22;
				offset.y = 10;
				y-=6;
				isCrouching = false;
			}
		}
		
		public function die(replay:Boolean=false):void
		{
			this.kill();
			icon.kill();
		}
		
		public function getIcon():ReplayIcon
		{
			return this.icon;
		}
		
		public function walkSpeed():Number { return WALK_SPEED; }
		
		public function getCanShoot():Boolean { return canShoot; }
	}
}