package
{
	/*
	 * Timequake Prototype Source Code. Released by @realnoyb on December 2012.
	 * This work is licensed under the Creative Commons Attribution-NonCommercial 3.0 Unported License. 
	 * To view a copy of this license, visit http://creativecommons.org/licenses/by-nc/3.0/.
	 */
	
	import org.flixel.*;

	public class PlayState extends FlxState
	{		
		
		[Embed(source="../data/block.png")] protected var ImgBlock:Class;
		[Embed(source="../data/door.png")] protected var ImgDoor:Class;
		/*[Embed(source="../levels/test.oel", mimeType="application/octet-stream")] protected var Level1:Class;
		[Embed(source="../levels/test2.oel", mimeType="application/octet-stream")] protected var Level2:Class;
		[Embed(source="../levels/test3.oel", mimeType="application/octet-stream")] protected var Level3:Class;
		[Embed(source="../levels/test4.oel", mimeType="application/octet-stream")] protected var Level4:Class;
		[Embed(source="../levels/test5.oel", mimeType="application/octet-stream")] protected var Level5:Class;
		*/
		private var _blocks:FlxGroup;
		private var _bullets:FlxGroup;
		private var _enemyBullets:FlxGroup;
		private var _dumbEnemies:FlxGroup;
		private var _descriptions:FlxGroup;
		
		private var _player:Player;
		private var _exit:FlxSprite;
		
		private static var isRerun:Boolean = false;
		private static var currentLevel:uint = 0;

		private static var _replay:Replay;
		private static var _replays:Array;
		
		private static var _frameNum:uint;
		
		private static var _accumulator:Number;
		private static const TIMESTEP:Number = 1.0/60.0;
		
		private var playerDead:Boolean;
		private var beatLevel:Boolean;
		private var lastHolster:Boolean;
		
		private var tempEnemy:DumbEnemy;
		
		override public function create():void
		{
			//Background
			var bg:FlxSprite = new FlxSprite(0,0,null);
			bg.createGraphic(FlxG.width, FlxG.height,0xffaaaaaa);
			add(bg);
			
			//Build Level
			//var bytes:ByteArray = new Level3;
			//var xml:XML = new XML(bytes.readUTFBytes(bytes.length));
			var xml:XML = LevelData.getLevelXMLData(currentLevel);
			loadXML(xml);
			
			//Replay logic
			if(!isRerun)
				_replay = new Replay(0,3,0);
			else
				_replay = _replays[currentLevel];
				
			if(_replays == null)
			{
				_replays = new Array(LevelData.NUM_LEVELS);
			}
				
			_frameNum = 0;
			_accumulator = 0;
			
				
			playerDead = false;
			beatLevel = false;
			lastHolster = false;
			
			FlxG.flash.start(0xff000000,0.1);
		}
		
		public function loadXML(myXML:XML):void
		{
			/*var myXML:XML;
			var myLoader:URLLoader = new URLLoader();
			myLoader.load(new URLRequest(filePath));
			
			// Call extractXMLFileData function if the XML file does load. 
			myLoader.addEventListener(Event.COMPLETE, extractXMLFileData); 
			
			// Call errorXMLFileData function if the XML file didn't load. 
			myLoader.addEventListener(IOErrorEvent.IO_ERROR, errorXMLFileData);
			
			function extractXMLFileData(event:Event):void { */
				//FlxG.log(myXMLLoader.data);    
				//myXML = new XML(event.target.data);
				
				makeBlocks(myXML.floors.rect);
				makeExit(myXML.actors.door);
				makePlayer(myXML.actors.player);
				makeEnemies(myXML.actors.enemy);
				add(_descriptions);
			/*} 
			
			// This function takes action if the XML file didn't load. 
			function errorXMLFileData(event:Event):void 
			{     
				FlxG.log("Error! The XML file didn't load! " + event); 
			}	*/
		}
		
		public function makePlayer(playerData:XMLList):void
		{
			//Bullets
			_bullets = new FlxGroup();
			for(var i:uint = 0; i < 20; i++)
			{	
				var bullet:FlxSprite = new FlxSprite(-100,-100,null);
				bullet.createGraphic(4,2,0xff000000);
				bullet.height = 1;
				bullet.exists = false;
				bullet.dead = true;
				_bullets.add(bullet);
			} 
			
			add(_bullets);
			
			//Entrance door
			var enterDoor:FlxSprite = new FlxSprite(playerData.@x, playerData.@y,ImgDoor);
			add(enterDoor);
			
			//Player
			var icon:ReplayIcon = new ReplayIcon(-100,-100);
			_player = new Player(playerData.@x, playerData.@y, _bullets, icon);
			add(_player);
			
			//Show icon during replays
			if(isRerun)
				add(icon);		
		}
		
		public function makeExit(doorData:XMLList):void
		{
			_exit = new FlxSprite(doorData.@x, doorData.@y, ImgDoor);
			add(_exit);
		}
		
		public function makeBlocks(blockData:XMLList):void
		{
			_blocks = new FlxGroup();
			
			for(var i:uint = 0; i < blockData.length(); i++)
			{
				var b:XML = blockData[i];
				
				var block:FlxTileblock = new FlxTileblock(b.@x, b.@y, b.@w, b.@h);
				block.loadGraphic(ImgBlock);
				_blocks.add(block);
			}
			
			add(_blocks);
			
		}
		
		public function makeEnemies(enemyData:XMLList):void
		{
			_dumbEnemies = new FlxGroup();
			_descriptions = new FlxGroup();
			_enemyBullets = new FlxGroup();
			
			for(var i:uint = 0; i < enemyData.length(); i++)
			{
				var ed:XML = enemyData[i];
				
				var name:String = ed.@name;
				
				//Enemy description
				var desc:String = ed.@name + ", age " + ed.@age + "\n" + ed.@description;
				var text:Description = new Description(ed.@x, ed.@y, 160, desc);
				text.setFormat(null, 8, 0xffff0000,"left",0xff000000);
				FlxG.log("Text: " + text.text);
				FlxG.log("Name: " + name);
				FlxG.log("Age: " + ed.@age);
				FlxG.log("Desc: " + ed.@description);
				FlxG.log("Dir: " + ed.@direction);
				text.visible = false;
				text.exists = false;
				text.dead = true;
				_descriptions.add(text);
				
				//Bullets
				var myBullets:FlxGroup = new FlxGroup();
				for(var j:uint = 0; j < 5; j++)
				{	
					var bullet:FlxSprite = new FlxSprite(-100,-100,null);
					bullet.createGraphic(4,2,0xff000000);
					bullet.height = 1;
					bullet.exists = false;
					bullet.dead = true;
					myBullets.add(bullet);
					_enemyBullets.add(bullet);
				} 
				
				//Enemy
				var e:DumbEnemy = new DumbEnemy(ed.@x, ed.@y, myBullets, text, ed.@AIType, ed.@direction == '0');
				_dumbEnemies.add(e);
			}
			
			add(_dumbEnemies);
			add(_enemyBullets);
		}
		
		
		override public function update():void
		{	
			
			if(beatLevel)
				return;
			
			if(_descriptions.countLiving() > 0)
			{
				_descriptions.update();
				return;
			}			
			
			//Fixed timestep hack
			_accumulator += FlxG.elapsed;
			
			if(_accumulator < TIMESTEP)
				return;
			
			_accumulator -= TIMESTEP;
			FlxG.elapsed = TIMESTEP;
				
			
			var canUpdate:Boolean = false;
			var numEnemies:uint = _dumbEnemies.countLiving();
			
			if(numEnemies == 0 && !beatLevel && FlxU.overlap(_player, _exit, blankFunction))
			{
				//Win level!
				winLevel();
			}
			
			if(playerDead)
			{
				canUpdate = true;
			}
			else if(!isRerun)
			{
				// First playthrough logic
				canUpdate = true;
				
				var actionPerformed:Boolean = false;
				
				if(FlxG.keys.justPressed("X"))
				{
					actionPerformed = _player.perform_action();
					if (actionPerformed && numEnemies > 0)
					{
						_replay.action();
					}
				}
				
				if(!actionPerformed && FlxG.keys.justPressed("C"))
				{
					actionPerformed = _player.perform_action2();
					if(actionPerformed)
					{
						_replay.action2();
						/*if(numEnemies > 0)
							_replay.action2();
						else if(!lastHolster && _player.getCanShoot())
						{
							_replay.action2();
						}
						else if (!lastHolster && !_player.getCanShoot())
						{
							lastHolster = true;
							_replay.action2();
						}*/
						
					}
				}
				
				if(FlxG.keys.justPressed("DOWN"))
				{
					actionPerformed = _player.moveDown();
					if (actionPerformed)
						_replay.stepDown();
				}
				
				if(FlxG.keys.justPressed("UP"))
				{
					actionPerformed = _player.moveUp();
					if (actionPerformed)
						_replay.stepUp();
				}
				
				if(!actionPerformed && FlxG.keys.LEFT && !FlxG.keys.RIGHT)
				{
					actionPerformed = _player.moveLeft();
					
					if(actionPerformed)
						_replay.stepLeft()
				}
				
				if(!actionPerformed && FlxG.keys.RIGHT && !FlxG.keys.LEFT)
				{
					actionPerformed = _player.moveRight();
					
					if(actionPerformed)
						_replay.stepRight();
				}
				
				if (!actionPerformed)
				{
					_player.wait();
					_replay.wait();
				}
				
				
				
			}
			else if (_frameNum <= _replay.size())
			{
				// Replay logic
				var currentAction:uint = _replay.getFrame(_frameNum);
				

				if(FlxG.keys.justPressed("X") && currentAction == Replay.ACTION)
				{
					_player.perform_action();
					canUpdate = true;
				}
				else if(FlxG.keys.justPressed("C") && currentAction == Replay.ACTION2)
				{
					_player.perform_action2();
					canUpdate = true;
				}
				else if((FlxG.keys.justPressed("DOWN") || !Replay.REPLAY_MOVEMENT) && currentAction == Replay.DOWN)
				{
					_player.moveDown();
					canUpdate = true;
				}
				else if((FlxG.keys.justPressed("UP")|| !Replay.REPLAY_MOVEMENT) && currentAction == Replay.UP)
				{
					_player.moveUp();
					canUpdate = true;
				}
				else if((FlxG.keys.LEFT && !FlxG.keys.RIGHT || !Replay.REPLAY_MOVEMENT) && currentAction == Replay.LEFT)
				{
					_player.moveLeft();
					canUpdate = true;
				}
				else if((FlxG.keys.RIGHT && !FlxG.keys.LEFT || !Replay.REPLAY_MOVEMENT) && currentAction == Replay.RIGHT)
				{
					_player.moveRight();
					canUpdate = true;
				}
				else if (currentAction == Replay.WAIT)
				{
					_player.wait();
					canUpdate = true;
				}
				
			}
			
			if(canUpdate)
			{
				// Update screen
				super.update();
				FlxU.collide(_player, _blocks);
				
				FlxU.overlap(_player, _dumbEnemies, squashEnemies);
				
				//Fix enemies so that the player can't move them.
				fixEnemies();
				FlxU.collide(_player, _dumbEnemies);
				unFixEnemies();
				
				FlxU.collide(_dumbEnemies, _blocks);
				FlxU.overlap(_bullets,_blocks,bulletHitBlock);
				FlxU.overlap(_enemyBullets,_blocks,bulletHitBlock);
				FlxU.overlap(_bullets, _dumbEnemies, bulletHitEnemy);
				FlxU.overlap(_enemyBullets, _player, bulletHitPlayer);
				_dumbEnemies.members.forEach(updateEnemySightLine);
				_frameNum++;
				var nextAction:uint = _replay.getFrame(_frameNum);
				_player.updateIcon(nextAction);
				
				if(PlayState.isRerun) FlxG.log(_frameNum + "/" + _replay.size() + " : " + _replay.getFrame(_frameNum));
			}
		}
		
		public function fixEnemies():void
		{
			var enemies:Array = _dumbEnemies.members;
			for(var i:int = 0; i < enemies.length; i++)
			{
				enemies[i].fixed = true;
				
				var e:DumbEnemy = enemies[i];
				if(e != null && e.dying && !e.dead)
					e.solid = false;
			}
		}
		
		public function unFixEnemies():void
		{
			var enemies:Array = _dumbEnemies.members;
			for(var i:int = 0; i < enemies.length; i++)
				enemies[i].fixed = false;
				
				var e:DumbEnemy = enemies[i];
				if(e != null && e.dying && !e.dead)
					e.solid = true;
		}
		
		public function winLevel():void
		{
			if(!PlayState.isRerun)
			{
				_replays[PlayState.currentLevel] = _replay;				
			}
			
			PlayState.currentLevel++;
			
			if(PlayState.currentLevel >= LevelData.NUM_LEVELS && !PlayState.isRerun)
			{
				PlayState.currentLevel = 0;
				PlayState.isRerun = true;
			}
			else if(PlayState.currentLevel >= LevelData.NUM_LEVELS && PlayState.isRerun)
			{
				PlayState.currentLevel--;
				return;
			}
			
			FlxG.fade.start(0xff000000,0.1,switchLevel);
			beatLevel = true;
		}
		
		public function switchLevel():void
		{
			FlxG.state = new PlayState();
		}
		
		public function updateEnemySightLine(e:DumbEnemy, index:int, array:Array):void
		{
			e.aboutToAlert = false;
			
			if(e.dead || !e.active)
				return;
			
			var canSee:Boolean = false;
			
			var topY:Number = e.topY();
			var bottomY:Number = e.y + e.height;
			var leftX:Number = e.x;
			var rightX:Number = e.x + e.width;
			
			var pTopY:Number = _player.y;
			var pBottomY:Number = _player.y + _player.height;
			var pLeftX:Number = _player.x;
			var pRightX:Number = _player.x + _player.width;
			
			if(e.facing == FlxSprite.RIGHT && pLeftX >= leftX && pBottomY >= topY && pTopY <= bottomY)
				canSee = true;
			else if(e.facing == FlxSprite.LEFT && pRightX <= rightX && pBottomY >= topY && pTopY <= bottomY)
				canSee = true;
			
			//Return false if enemy not facing right direction
			if(!canSee)
			{
				e.updateSightLine(false, false);
				return;
			}
			
			var closeTest:FlxSprite = new FlxSprite(leftX - 4, topY);
			closeTest.createGraphic(8 + e.width,1);
			tempEnemy = e;
			
			//Return false if enemies are bunched up too closely.
			if(FlxU.overlap(closeTest,_dumbEnemies,testProximity))
			{
				e.updateSightLine(false,false);
				return;
			}
				
			var topSight:FlxSprite, bottomSight:FlxSprite;
			var losWidth:Number;
			
			if(e.facing == FlxSprite.RIGHT)
			{
				losWidth = pRightX - leftX;
				
				topSight = new FlxSprite(leftX, topY);
				topSight.createGraphic(losWidth,1);
				
				bottomSight = new FlxSprite(leftX, bottomY - 4);
				bottomSight.createGraphic(losWidth, 1);
			}
			else
			{
				losWidth = rightX - pLeftX;
				topSight = new FlxSprite(pLeftX, topY);
				topSight.createGraphic(losWidth,1);
				
				bottomSight = new FlxSprite(pLeftX, bottomY - 4);
				bottomSight.createGraphic(losWidth,1);
			}
			
			var top:Boolean = false;
			var bottom:Boolean = false;
			
			if (FlxU.overlap(topSight,_player, blankFunction))
			{
				if(!FlxU.overlap(topSight, _blocks, blankFunction))
				{
					top = true;
				}
			}
			
			if (FlxU.overlap(bottomSight,_player, blankFunction))
			{
				if(!FlxU.overlap(bottomSight, _blocks, blankFunction))
				{
					bottom = true;
				}
			}
			
			e.aboutToAlert = top || bottom || (e.getState() == DumbEnemy.STATE_ALERT);
			
			e.updateSightLine(top, bottom);
		}
		
		public function blankFunction(s1:FlxObject, s2:FlxObject):Boolean
		{
			return true;
		}
		
		public function testProximity(s1:FlxObject, s2:DumbEnemy):Boolean
		{
			return s2 != tempEnemy && s2.aboutToAlert;
		}
		
		public function squashEnemies(player:Player, enemy:DumbEnemy):void
		{
			if(player.y + player.height < enemy.y + 4)
			{
				enemy.die(isRerun);
				player.y = enemy.y - player.height;
				//player.bounce();
			}
			else
			{
				//var deadSprite:DeadPlayer = new DeadPlayer(player.x - player.offset.x, player.y - player.offset.y);
				//deadSprite.facing = player.facing;
				//add(deadSprite);
				
				//player.die();
				//playerDead = true;
				
				
			}
		}
		
		public function bulletHitBlock(bullet:FlxSprite, block:FlxTileblock):void
		{
			bullet.kill();
		}
		
		public function bulletHitEnemy(bullet:FlxSprite, enemy:DumbEnemy):void
		{
			enemy.die(isRerun);
			bullet.kill();
		}
		
		public function bulletHitPlayer(bullet:FlxSprite, player:Player):void
		{
			FlxG.state = new PlayState();
		}
		
	}
	
}